@@ -1,6 +1,6 @@
 # SPDX-License-Identifier: LGPL-2.1-or-later
 
-# Copyright (C) 2020, 2021 igo95862
+# Copyright (C) 2020-2022 igo95862
 
 # This file is part of python-sdbus
 
@@ -18,7 +18,7 @@
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 
-from .dbus_common import (
+from .dbus_common_funcs import (
     get_default_bus,
     request_default_bus_name,
     request_default_bus_name_async,
@@ -57,21 +57,25 @@ from .dbus_exceptions import (
     DbusUnknownObjectError,
     DbusUnknownPropertyError,
 )
-from .dbus_proxy_async import (
+from .dbus_proxy_async_interfaces import (
     DbusInterfaceCommonAsync,
     DbusObjectManagerInterfaceAsync,
+)
+from .dbus_proxy_async_method import (
     dbus_method_async,
     dbus_method_async_override,
+)
+from .dbus_proxy_async_property import (
     dbus_property_async,
     dbus_property_async_override,
-    dbus_signal_async,
 )
-from .dbus_proxy_sync import (
+from .dbus_proxy_async_signal import dbus_signal_async
+from .dbus_proxy_sync_interfaces import (
     DbusInterfaceCommon,
     DbusObjectManagerInterface,
-    dbus_method,
-    dbus_property,
 )
+from .dbus_proxy_sync_method import dbus_method
+from .dbus_proxy_sync_property import dbus_property
 from .sd_bus_internals import (
     DbusDeprecatedFlag,
     DbusHiddenFlag,
@@ -88,6 +92,7 @@ from .sd_bus_internals import (
     SdBusUnmappedMessageError,
     decode_object_path,
     encode_object_path,
+    map_exception_to_dbus_error,
     sd_bus_open,
     sd_bus_open_system,
     sd_bus_open_system_machine,
@@ -96,7 +101,7 @@ from .sd_bus_internals import (
     sd_bus_open_user_machine,
 )
 
-__all__ = [
+__all__ = (
     'get_default_bus', 'request_default_bus_name',
     'request_default_bus_name_async', 'set_default_bus',
 
@@ -121,14 +126,22 @@ __all__ = [
     'DbusUnknownMethodError', 'DbusUnknownObjectError',
     'DbusUnknownPropertyError',
 
+    'DbusInterfaceCommonAsync',
     'DbusObjectManagerInterfaceAsync',
-    'DbusInterfaceCommonAsync', 'dbus_method_async',
-    'dbus_method_async_override', 'dbus_property_async',
-    'dbus_property_async_override', 'dbus_signal_async',
+
+    'dbus_method_async',
+    'dbus_method_async_override',
+
+    'dbus_property_async',
+    'dbus_property_async_override',
+
+    'dbus_signal_async',
 
     'DbusInterfaceCommon',
     'DbusObjectManagerInterface',
+
     'dbus_method',
+
     'dbus_property',
 
     'DbusDeprecatedFlag',
@@ -146,10 +159,11 @@ __all__ = [
     'SdBusUnmappedMessageError',
     'decode_object_path',
     'encode_object_path',
+    'map_exception_to_dbus_error',
     'sd_bus_open',
     'sd_bus_open_system',
     'sd_bus_open_system_machine',
     'sd_bus_open_system_remote',
     'sd_bus_open_user',
     'sd_bus_open_user_machine',
-]
+)
