@@ -23,7 +23,7 @@
 // TODO: adding interface to different buses, recalculating vtable
 
 static int SdBusInterface_init(SdBusInterfaceObject* self, PyObject* Py_UNUSED(args), PyObject* Py_UNUSED(kwds)) {
-        self->interface_slot = (SdBusSlotObject*)CALL_PYTHON_CHECK_RETURN_NEG1(PyObject_CallFunctionObjArgs(SdBusSlot_class, NULL));
+        self->interface_slot = (SdBusSlotObject*)CALL_PYTHON_CHECK_RETURN_NEG1(SD_BUS_PY_CLASS_DUNDER_NEW(SdBusSlot_class));
         self->method_list = CALL_PYTHON_CHECK_RETURN_NEG1(PyList_New((Py_ssize_t)0));
         self->method_dict = CALL_PYTHON_CHECK_RETURN_NEG1(PyDict_New());
         self->property_list = CALL_PYTHON_CHECK_RETURN_NEG1(PyList_New((Py_ssize_t)0));
@@ -49,7 +49,7 @@ static void SdBusInterface_dealloc(SdBusInterfaceObject* self) {
         SD_BUS_DEALLOC_TAIL;
 }
 
-inline int _check_callable_or_none(PyObject* some_object) {
+static inline int _check_callable_or_none(PyObject* some_object) {
         return PyCallable_Check(some_object) || (Py_None == some_object);
 }
 
@@ -58,8 +58,8 @@ static PyObject* SdBusInterface_add_property(SdBusInterfaceObject* self, PyObjec
         // Arguments
         // Name, Signature, Get, Set, Flags
         SD_BUS_PY_CHECK_ARGS_NUMBER(5);
-        SD_BUS_PY_CHECK_ARG_TYPE(0, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(1, PyUnicode_Type);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(1, PyUnicode_Check);
         SD_BUS_PY_CHECK_ARG_CHECK_FUNC(2, PyCallable_Check);
         SD_BUS_PY_CHECK_ARG_CHECK_FUNC(3, _check_callable_or_none);
         SD_BUS_PY_CHECK_ARG_CHECK_FUNC(4, PyLong_Check);
@@ -96,10 +96,10 @@ static PyObject* SdBusInterface_add_method(SdBusInterfaceObject* self, PyObject*
         // Method name, signature, names of input values, result signature,
         // names of result values, flags, callback function or coroutine
         SD_BUS_PY_CHECK_ARGS_NUMBER(7);
-        SD_BUS_PY_CHECK_ARG_TYPE(0, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(1, PyUnicode_Type);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(1, PyUnicode_Check);
         SD_BUS_PY_CHECK_ARG_CHECK_FUNC(2, PySequence_Check);
-        SD_BUS_PY_CHECK_ARG_TYPE(3, PyUnicode_Type);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(3, PyUnicode_Check);
         SD_BUS_PY_CHECK_ARG_CHECK_FUNC(4, PySequence_Check);
         SD_BUS_PY_CHECK_ARG_CHECK_FUNC(5, PyLong_Check);
         SD_BUS_PY_CHECK_ARG_CHECK_FUNC(6, PyCallable_Check);
@@ -153,8 +153,8 @@ static PyObject* SdBusInterface_add_signal(SdBusInterfaceObject* self, PyObject*
         // Arguments
         // Signal name, signature, names of input values, flags
         SD_BUS_PY_CHECK_ARGS_NUMBER(4);
-        SD_BUS_PY_CHECK_ARG_TYPE(0, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(1, PyUnicode_Type);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(1, PyUnicode_Check);
         SD_BUS_PY_CHECK_ARG_CHECK_FUNC(2, PySequence_Check);
         SD_BUS_PY_CHECK_ARG_CHECK_FUNC(3, PyLong_Check);
 
@@ -343,6 +343,7 @@ PyType_Spec SdBusInterfaceType = {
     .flags = Py_TPFLAGS_DEFAULT,
     .slots =
         (PyType_Slot[]){
+            {Py_tp_new, PyType_GenericNew},
             {Py_tp_init, (initproc)SdBusInterface_init},
             {Py_tp_dealloc, (destructor)SdBusInterface_dealloc},
             {Py_tp_methods, SdBusInterface_methods},
@@ -351,54 +352,52 @@ PyType_Spec SdBusInterfaceType = {
         },
 };
 
+static int set_dbus_error_from_python_exception(sd_bus_error* ret_error) {
+#ifdef Py_LIMITED_API
+        PyObject* dbus_error_bytes CLEANUP_PY_OBJECT = NULL;
+#endif
+        PyObject* current_exception = PyErr_Occurred();
+        if (NULL == current_exception) {
+                goto fail;
+        }
+        PyObject* dbus_error_str = CALL_PYTHON_GOTO_FAIL(PyDict_GetItem(exception_to_dbus_error_dict, current_exception));
+#ifndef Py_LIMITED_API
+        const char* dbus_error_char_ptr = SD_BUS_PY_UNICODE_AS_CHAR_PTR_GOTO_FAIL(dbus_error_str);
+#else
+        dbus_error_bytes = SD_BUS_PY_UNICODE_AS_BYTES_GOTO_FAIL(dbus_error_str);
+        const char* dbus_error_char_ptr = SD_BUS_PY_BYTES_AS_CHAR_PTR_GOTO_FAIL(dbus_error_bytes);
+#endif
+        return sd_bus_error_set(ret_error, dbus_error_char_ptr, "");
+fail:
+        return sd_bus_error_set(ret_error, SD_BUS_ERROR_FAILED, "");
+}
+
+#define METHOD_CALLBACK_ERROR_CHECK(py_function) CALL_PYTHON_FAIL_ACTION(py_function, return set_dbus_error_from_python_exception(ret_error))
+
 static int _SdBusInterface_callback(sd_bus_message* m, void* userdata, sd_bus_error* ret_error) {
         // TODO: Better error handling
         SdBusInterfaceObject* self = userdata;
         // Get the member name from the message
         const char* member_char_ptr = sd_bus_message_get_member(m);
-        PyObject* member_name_bytes CLEANUP_PY_OBJECT = PyBytes_FromString(member_char_ptr);
-        PyObject* callback_object = PyDict_GetItem(self->method_dict, member_name_bytes);
-        if (callback_object == NULL) {
-                sd_bus_error_set(ret_error, SD_BUS_ERROR_UNKNOWN_METHOD, "");
-                return -1;
-        };
+        PyObject* member_name_bytes CLEANUP_PY_OBJECT = METHOD_CALLBACK_ERROR_CHECK(PyBytes_FromString(member_char_ptr));
+        PyObject* callback_object = METHOD_CALLBACK_ERROR_CHECK(PyDict_GetItem(self->method_dict, member_name_bytes));
 
-        PyObject* running_loop CLEANUP_PY_OBJECT = PyObject_CallFunctionObjArgs(asyncio_get_running_loop, NULL);
-        if (running_loop == NULL) {
-                sd_bus_error_set(ret_error, SD_BUS_ERROR_FAILED, "");
-                return -1;
-        }
+        PyObject* running_loop CLEANUP_PY_OBJECT = METHOD_CALLBACK_ERROR_CHECK(PyObject_CallFunctionObjArgs(asyncio_get_running_loop, NULL));
+
+        PyObject* new_message CLEANUP_PY_OBJECT = METHOD_CALLBACK_ERROR_CHECK(SD_BUS_PY_CLASS_DUNDER_NEW(SdBusMessage_class));
 
-        PyObject* new_message CLEANUP_PY_OBJECT = PyObject_CallFunctionObjArgs(SdBusMessage_class, NULL);
-        if (new_message == NULL) {
-                sd_bus_error_set(ret_error, SD_BUS_ERROR_FAILED, "");
-                return -1;
-        }
         _SdBusMessage_set_messsage((SdBusMessageObject*)new_message, m);
 
-        PyObject* is_coroutine_test_object CLEANUP_PY_OBJECT = PyObject_CallFunctionObjArgs(is_coroutine_function, callback_object, NULL);
-        if (is_coroutine_test_object == NULL) {
-                return -1;
-        }
+        PyObject* is_coroutine_test_object CLEANUP_PY_OBJECT =
+            METHOD_CALLBACK_ERROR_CHECK(PyObject_CallFunctionObjArgs(is_coroutine_function, callback_object, NULL));
 
         if (Py_True == is_coroutine_test_object) {
                 // Create coroutine
-                PyObject* coroutine_activated CLEANUP_PY_OBJECT = PyObject_CallFunctionObjArgs(callback_object, new_message, NULL);
-                if (coroutine_activated == NULL) {
-                        return -1;
-                }
+                PyObject* coroutine_activated CLEANUP_PY_OBJECT = METHOD_CALLBACK_ERROR_CHECK(PyObject_CallFunctionObjArgs(callback_object, new_message, NULL));
 
-                PyObject* task CLEANUP_PY_OBJECT = PyObject_CallMethodObjArgs(running_loop, create_task_str, coroutine_activated, NULL);
-                if (task == NULL) {
-                        sd_bus_error_set(ret_error, SD_BUS_ERROR_FAILED, "");
-                        return -1;
-                }
+                Py_XDECREF(METHOD_CALLBACK_ERROR_CHECK(PyObject_CallMethodObjArgs(running_loop, create_task_str, coroutine_activated, NULL)));
         } else {
-                PyObject* handle CLEANUP_PY_OBJECT = PyObject_CallMethodObjArgs(running_loop, call_soon_str, callback_object, new_message, NULL);
-                if (handle == NULL) {
-                        sd_bus_error_set(ret_error, SD_BUS_ERROR_FAILED, "");
-                        return -1;
-                }
+                Py_XDECREF(METHOD_CALLBACK_ERROR_CHECK(PyObject_CallFunctionObjArgs(callback_object, new_message, NULL)));
         }
 
         sd_bus_error_set(ret_error, NULL, NULL);
@@ -412,16 +411,22 @@ static int _SdBusInterface_property_get_callback(sd_bus* Py_UNUSED(bus),
                                                  const char* property,
                                                  sd_bus_message* reply,
                                                  void* userdata,
-                                                 sd_bus_error* Py_UNUSED(ret_error)) {
+                                                 sd_bus_error* ret_error) {
         SdBusInterfaceObject* self = userdata;
-        PyObject* property_name_bytes CLEANUP_PY_OBJECT = PyBytes_FromString(property);
-        PyObject* get_call = CALL_PYTHON_CHECK_RETURN_NEG1(PyDict_GetItem(self->property_get_dict, property_name_bytes));
+        PyObject* property_name_bytes CLEANUP_PY_OBJECT = NULL;
+        PyObject* get_call = NULL;
+        PyObject* new_message CLEANUP_PY_OBJECT = NULL;
 
-        PyObject* new_message CLEANUP_PY_OBJECT = CALL_PYTHON_CHECK_RETURN_NEG1(PyObject_CallFunctionObjArgs(SdBusMessage_class, NULL));
+        property_name_bytes = CALL_PYTHON_GOTO_FAIL(PyBytes_FromString(property));
+        get_call = CALL_PYTHON_GOTO_FAIL(PyDict_GetItem(self->property_get_dict, property_name_bytes));
+
+        new_message = CALL_PYTHON_GOTO_FAIL(SD_BUS_PY_CLASS_DUNDER_NEW(SdBusMessage_class));
         _SdBusMessage_set_messsage((SdBusMessageObject*)new_message, reply);
 
-        Py_XDECREF(CALL_PYTHON_CHECK_RETURN_NEG1(PyObject_CallFunctionObjArgs(get_call, new_message, NULL)));
+        Py_XDECREF(CALL_PYTHON_GOTO_FAIL(PyObject_CallFunctionObjArgs(get_call, new_message, NULL)));
         return 0;
+fail:
+        return set_dbus_error_from_python_exception(ret_error);
 }
 
 static int _SdBusInterface_property_set_callback(sd_bus* Py_UNUSED(bus),
@@ -430,14 +435,21 @@ static int _SdBusInterface_property_set_callback(sd_bus* Py_UNUSED(bus),
                                                  const char* property,
                                                  sd_bus_message* value,
                                                  void* userdata,
-                                                 sd_bus_error* Py_UNUSED(ret_error)) {
+                                                 sd_bus_error* ret_error) {
         SdBusInterfaceObject* self = userdata;
-        PyObject* property_name_bytes CLEANUP_PY_OBJECT = PyBytes_FromString(property);
-        PyObject* set_call = CALL_PYTHON_CHECK_RETURN_NEG1(PyDict_GetItem(self->property_set_dict, property_name_bytes));
+        PyObject* property_name_bytes CLEANUP_PY_OBJECT = NULL;
+        PyObject* new_message CLEANUP_PY_OBJECT = NULL;
+        property_name_bytes = PyBytes_FromString(property);
+        if (NULL == property_name_bytes) {
+                goto fail;
+        }
+        PyObject* set_call = CALL_PYTHON_GOTO_FAIL(PyDict_GetItem(self->property_set_dict, property_name_bytes));
 
-        PyObject* new_message CLEANUP_PY_OBJECT = CALL_PYTHON_CHECK_RETURN_NEG1(PyObject_CallFunctionObjArgs(SdBusMessage_class, NULL));
+        new_message = CALL_PYTHON_GOTO_FAIL(SD_BUS_PY_CLASS_DUNDER_NEW(SdBusMessage_class));
         _SdBusMessage_set_messsage((SdBusMessageObject*)new_message, value);
 
-        Py_XDECREF(CALL_PYTHON_CHECK_RETURN_NEG1(PyObject_CallFunctionObjArgs(set_call, new_message, NULL)));
+        Py_XDECREF(CALL_PYTHON_GOTO_FAIL(PyObject_CallFunctionObjArgs(set_call, new_message, NULL)));
         return 0;
+fail:
+        return set_dbus_error_from_python_exception(ret_error);
 }
