@@ -20,11 +20,6 @@
 */
 #include "sd_bus_internals.h"
 
-static int SdBusMessage_init(SdBusMessageObject* self, PyObject* Py_UNUSED(args), PyObject* Py_UNUSED(kwds)) {
-        self->message_ref = NULL;
-        return 0;
-}
-
 void _SdBusMessage_set_messsage(SdBusMessageObject* self, sd_bus_message* new_message) {
         self->message_ref = sd_bus_message_ref(new_message);
 }
@@ -611,7 +606,7 @@ static PyObject* SdBusMessage_append_data(SdBusMessageObject* self, PyObject* co
                 PyErr_SetString(PyExc_TypeError, "Minimum 2 args required");
                 return NULL;
         }
-        SD_BUS_PY_CHECK_ARG_TYPE(0, PyUnicode_Type);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, PyUnicode_Check);
 
         const char* signature_char_ptr = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[0]);
 
@@ -653,8 +648,8 @@ static PyObject* SdBusMessage_append_data(SdBusMessageObject* self, PyObject* ar
 #ifndef Py_LIMITED_API
 static PyObject* SdBusMessage_open_container(SdBusMessageObject* self, PyObject* const* args, Py_ssize_t nargs) {
         SD_BUS_PY_CHECK_ARGS_NUMBER(2);
-        SD_BUS_PY_CHECK_ARG_TYPE(0, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(1, PyUnicode_Type);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(1, PyUnicode_Check);
 
         const char* container_type_char_ptr = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[0]);
         const char* container_contents_char_ptr = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[1]);
@@ -678,8 +673,8 @@ static PyObject* SdBusMessage_close_container(SdBusMessageObject* self, PyObject
 #ifndef Py_LIMITED_API
 static PyObject* SdBusMessage_enter_container(SdBusMessageObject* self, PyObject* const* args, Py_ssize_t nargs) {
         SD_BUS_PY_CHECK_ARGS_NUMBER(2);
-        SD_BUS_PY_CHECK_ARG_TYPE(0, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(1, PyUnicode_Type);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(1, PyUnicode_Check);
 
         const char* container_type_char_ptr = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[0]);
         const char* container_contents_char_ptr = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[1]);
@@ -702,7 +697,7 @@ static PyObject* SdBusMessage_exit_container(SdBusMessageObject* self, PyObject*
 
 static SdBusMessageObject* SdBusMessage_create_reply(SdBusMessageObject* self, PyObject* Py_UNUSED(args)) {
         SdBusMessageObject* new_reply_message CLEANUP_SD_BUS_MESSAGE =
-            (SdBusMessageObject*)CALL_PYTHON_AND_CHECK(PyObject_CallFunctionObjArgs(SdBusMessage_class, NULL));
+            (SdBusMessageObject*)CALL_PYTHON_AND_CHECK(SD_BUS_PY_CLASS_DUNDER_NEW(SdBusMessage_class));
 
         CALL_SD_BUS_AND_CHECK(sd_bus_message_new_method_return(self->message_ref, &new_reply_message->message_ref));
 
@@ -984,20 +979,11 @@ static PyObject* SdBusMessage_get_contents2(SdBusMessageObject* self, PyObject*
         return iter_tuple_or_single(&read_parser);
 }
 
-static PyObject* SdBusMessage_get_member(SdBusMessageObject* self, PyObject* Py_UNUSED(args)) {
-        const char* member_char_ptr = sd_bus_message_get_member(self->message_ref);
-        if (member_char_ptr == NULL) {
-                PyErr_SetString(PyExc_RuntimeError, "Failed to get message member field");
-                return NULL;
-        }
-        return PyUnicode_FromString(member_char_ptr);
-}
-
 #ifndef Py_LIMITED_API
 static SdBusMessageObject* SdBusMessage_create_error_reply(SdBusMessageObject* self, PyObject* const* args, Py_ssize_t nargs) {
         SD_BUS_PY_CHECK_ARGS_NUMBER(2);
-        SD_BUS_PY_CHECK_ARG_TYPE(0, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(1, PyUnicode_Type);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(1, PyUnicode_Check);
 
         const char* name = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[0]);
         const char* error_message = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[1]);
@@ -1008,7 +994,7 @@ static SdBusMessageObject* SdBusMessage_create_error_reply(SdBusMessageObject* s
         CALL_PYTHON_BOOL_CHECK(PyArg_ParseTuple(args, "ss", &name, &error_message, NULL));
 #endif
         SdBusMessageObject* new_reply_message CLEANUP_SD_BUS_MESSAGE =
-            (SdBusMessageObject*)CALL_PYTHON_AND_CHECK(PyObject_CallFunctionObjArgs(SdBusMessage_class, NULL));
+            (SdBusMessageObject*)CALL_PYTHON_AND_CHECK(SD_BUS_PY_CLASS_DUNDER_NEW(SdBusMessage_class));
 
         CALL_SD_BUS_AND_CHECK(sd_bus_message_new_method_errorf(self->message_ref, &new_reply_message->message_ref, name, "%s", error_message));
 
@@ -1025,7 +1011,6 @@ static PyMethodDef SdBusMessage_methods[] = {
     {"dump", (PyCFunction)SdBusMessage_dump, METH_NOARGS, "Dump message to stdout"},
     {"seal", (PyCFunction)SdBusMessage_seal, METH_NOARGS, "Seal message contents"},
     {"get_contents", (PyCFunction)SdBusMessage_get_contents2, METH_NOARGS, "Iterate over message contents"},
-    {"get_member", (PyCFunction)SdBusMessage_get_member, METH_NOARGS, "Get message member field"},
     {"create_reply", (PyCFunction)SdBusMessage_create_reply, METH_NOARGS, "Create reply message"},
     {"create_error_reply", (SD_BUS_PY_FUNC_TYPE)SdBusMessage_create_error_reply, SD_BUS_PY_METH, "Create error reply with error name and error message"},
     {"send", (PyCFunction)SdBusMessage_send, METH_NOARGS, "Queue message to be sent"},
@@ -1052,8 +1037,48 @@ static int SdBusMessage_expect_reply_setter(SdBusMessageObject* self, PyObject*
         return 0;
 }
 
+static PyObject* SdBusMessage_destination_getter(SdBusMessageObject* self, void* Py_UNUSED(closure)) {
+        const char* destination_char_ptr = sd_bus_message_get_destination(self->message_ref);
+        if (NULL != destination_char_ptr) {
+                return PyUnicode_FromString(destination_char_ptr);
+        } else {
+                Py_RETURN_NONE;
+        }
+}
+
+static PyObject* SdBusMessage_path_getter(SdBusMessageObject* self, void* Py_UNUSED(closure)) {
+        const char* path_char_ptr = sd_bus_message_get_path(self->message_ref);
+        if (NULL != path_char_ptr) {
+                return PyUnicode_FromString(path_char_ptr);
+        } else {
+                Py_RETURN_NONE;
+        }
+}
+
+static PyObject* SdBusMessage_interface_getter(SdBusMessageObject* self, void* Py_UNUSED(closure)) {
+        const char* interface_char_ptr = sd_bus_message_get_interface(self->message_ref);
+        if (NULL != interface_char_ptr) {
+                return PyUnicode_FromString(interface_char_ptr);
+        } else {
+                Py_RETURN_NONE;
+        }
+}
+
+static PyObject* SdBusMessage_member_getter(SdBusMessageObject* self, void* Py_UNUSED(closure)) {
+        const char* member_char_ptr = sd_bus_message_get_member(self->message_ref);
+        if (NULL != member_char_ptr) {
+                return PyUnicode_FromString(member_char_ptr);
+        } else {
+                Py_RETURN_NONE;
+        }
+}
+
 static PyGetSetDef SdBusMessage_properies[] = {
     {"expect_reply", (getter)SdBusMessage_expect_reply_getter, (setter)SdBusMessage_expect_reply_setter, "Expect reply message?", NULL},
+    {"destination", (getter)SdBusMessage_destination_getter, NULL, "Message destination service name", NULL},
+    {"path", (getter)SdBusMessage_path_getter, NULL, "Message destination object path", NULL},
+    {"interface", (getter)SdBusMessage_interface_getter, NULL, "Message destination interface name", NULL},
+    {"member", (getter)SdBusMessage_member_getter, NULL, "Message destination member name", NULL},
     {0},
 };
 
@@ -1064,7 +1089,7 @@ PyType_Spec SdBusMessageType = {
     .flags = Py_TPFLAGS_DEFAULT,
     .slots =
         (PyType_Slot[]){
-            {Py_tp_init, (initproc)SdBusMessage_init},
+            {Py_tp_new, PyType_GenericNew},
             {Py_tp_dealloc, (destructor)SdBusMessage_dealloc},
             {Py_tp_methods, SdBusMessage_methods},
             {Py_tp_getset, SdBusMessage_properies},
