@@ -21,9 +21,7 @@ from __future__ import annotations
 
 from typing import List, Optional
 
-from sdbus.sd_bus_internals import SdBus
-
-from sdbus import DbusInterfaceCommon, dbus_method, dbus_property
+from sdbus import DbusInterfaceCommon, SdBus, dbus_method, dbus_property
 
 
 class FreedesktopDbus(DbusInterfaceCommon,
