@@ -20,19 +20,20 @@
 
 from __future__ import annotations
 
-from asyncio import Event, get_running_loop, wait_for
+from asyncio import Event, get_running_loop, sleep, wait_for
 from asyncio.subprocess import create_subprocess_exec
 from typing import Tuple
 from unittest import SkipTest
 
-from sdbus.dbus_common import PROPERTY_FLAGS_MASK, count_bits
+from sdbus.dbus_common_funcs import PROPERTY_FLAGS_MASK, count_bits
 from sdbus.sd_bus_internals import (
     DBUS_ERROR_TO_EXCEPTION,
     DbusDeprecatedFlag,
     DbusPropertyConstFlag,
     DbusPropertyEmitsChangeFlag,
-    SdBus,
+    is_interface_name_valid,
 )
+from sdbus.unittest import IsolatedDbusTestCase
 
 from sdbus import (
     DbusFailedError,
@@ -40,6 +41,7 @@ from sdbus import (
     DbusInterfaceCommonAsync,
     DbusNoReplyFlag,
     DbusUnknownObjectError,
+    SdBusLibraryError,
     SdBusUnmappedMessageError,
     dbus_method_async,
     dbus_method_async_override,
@@ -48,10 +50,8 @@ from sdbus import (
     dbus_signal_async,
 )
 
-from .common_test_util import TempDbusTest
 
-
-class TestPing(TempDbusTest):
+class TestPing(IsolatedDbusTestCase):
 
     async def test_ping_with_busctl(self) -> None:
         try:
@@ -77,7 +77,7 @@ class TestPing(TempDbusTest):
         self.assertIsNone(r.get_contents())
 
 
-class TestRequestName(TempDbusTest):
+class TestRequestName(IsolatedDbusTestCase):
     async def test_request_name(self) -> None:
         await self.bus.request_name_async("org.example.test", 0)
 
@@ -188,6 +188,10 @@ class TestInterface(DbusInterfaceCommonAsync,
     async def test_struct_return_workaround(self) -> Tuple[Tuple[str, str]]:
         return (('hello', 'world'), )
 
+    @dbus_method_async()
+    async def looong_method(self) -> None:
+        await sleep(100)
+
 
 class DbusErrorTest(DbusFailedError):
     dbus_error_name = 'org.example.Error'
@@ -197,30 +201,23 @@ class DbusErrorUnmappedLater(DbusFailedError):
     dbus_error_name = 'org.example.Nothing'
 
 
-def initialize_object(bus: SdBus) -> Tuple[TestInterface, TestInterface]:
+def initialize_object() -> Tuple[TestInterface, TestInterface]:
     test_object = TestInterface()
-    test_object.export_to_dbus('/', bus)
+    test_object.export_to_dbus('/')
 
     test_object_connection = TestInterface.new_proxy(
-        "org.example.test", '/', bus)
+        "org.example.test", '/')
 
     return test_object, test_object_connection
 
 
-class TestProxy(TempDbusTest):
+class TestProxy(IsolatedDbusTestCase):
     async def asyncSetUp(self) -> None:
         await super().asyncSetUp()
         await self.bus.request_name_async("org.example.test", 0)
 
-        test_object = TestInterface()
-        test_object.export_to_dbus('/', self.bus)
-        test_object_connection = TestInterface.new_proxy(
-            "org.example.test", '/', self.bus)
-
-        await test_object_connection.dbus_ping()
-
     async def test_method_kwargs(self) -> None:
-        test_object, test_object_connection = initialize_object(self.bus)
+        test_object, test_object_connection = initialize_object()
 
         self.assertEqual(
             'TEST',
@@ -256,7 +253,7 @@ class TestProxy(TempDbusTest):
                 input='ASD', is_upper=False))
 
     async def test_method(self) -> None:
-        test_object, test_object_connection = initialize_object(self.bus)
+        test_object, test_object_connection = initialize_object()
 
         test_string = 'asdafsrfgdrtuhrytuj'
 
@@ -282,7 +279,7 @@ class TestProxy(TempDbusTest):
         )
 
     async def test_subclass(self) -> None:
-        test_object, test_object_connection = initialize_object(self.bus)
+        test_object, test_object_connection = initialize_object()
 
         test_var = ['asdasd']
 
@@ -327,7 +324,7 @@ class TestProxy(TempDbusTest):
         self.assertRaises(TypeError, bad_call)
 
     async def test_properties(self) -> None:
-        test_object, test_object_connection = initialize_object(self.bus)
+        test_object, test_object_connection = initialize_object()
 
         self.assertEqual(
             'test_property',
@@ -365,7 +362,7 @@ class TestProxy(TempDbusTest):
         )
 
     async def test_signal(self) -> None:
-        test_object, test_object_connection = initialize_object(self.bus)
+        test_object, test_object_connection = initialize_object()
 
         loop = get_running_loop()
 
@@ -382,7 +379,7 @@ class TestProxy(TempDbusTest):
         self.assertEqual(test_tuple, await wait_for(q.get(), timeout=1))
 
     async def test_exceptions(self) -> None:
-        test_object, test_object_connection = initialize_object(self.bus)
+        test_object, test_object_connection = initialize_object()
 
         async def first_test() -> None:
             await test_object_connection.raise_base_exception()
@@ -447,7 +444,7 @@ class TestProxy(TempDbusTest):
         self.assertRaises(SdBusUnmappedMessageError, t4.result)
 
     async def test_no_reply_method(self) -> None:
-        test_object, test_object_connection = initialize_object(self.bus)
+        test_object, test_object_connection = initialize_object()
 
         await wait_for(test_object_connection.no_reply_method('yes'),
                        timeout=1)
@@ -455,7 +452,7 @@ class TestProxy(TempDbusTest):
         await wait_for(test_object.no_reply_sync.wait(), timeout=1)
 
     async def test_interface_remove(self) -> None:
-        test_object, test_object_connection = initialize_object(self.bus)
+        test_object, test_object_connection = initialize_object()
 
         from gc import collect
 
@@ -475,7 +472,7 @@ class TestProxy(TempDbusTest):
         self.assertRaises(DbusUnknownObjectError, t.result)
 
     def test_docstring(self) -> None:
-        test_object, test_object_connection = initialize_object(self.bus)
+        test_object, test_object_connection = initialize_object()
 
         from pydoc import getdoc
 
@@ -494,7 +491,7 @@ class TestProxy(TempDbusTest):
             getdoc(test_object_connection.test_signal))
 
     async def test_emits_properties_changed(self) -> None:
-        test_object, test_object_connection = initialize_object(self.bus)
+        test_object, test_object_connection = initialize_object()
 
         test_str = 'should_be_emited'
 
@@ -557,7 +554,11 @@ class TestProxy(TempDbusTest):
                 def test_constant(self) -> str:
                     return "a"
 
-        self.assertRaises(ValueError, must_raise_value_error)
+        self.assertRaisesRegex(
+            AssertionError,
+            '^Incorrect number of Property flags',
+            must_raise_value_error,
+        )
 
         def should_be_no_error() -> None:
             class ValidPropertiesFlags(
@@ -571,3 +572,134 @@ class TestProxy(TempDbusTest):
                     return "a"
 
         should_be_no_error()
+
+    async def test_bus_close(self) -> None:
+        test_object, test_object_connection = initialize_object()
+
+        loop = get_running_loop()
+
+        async def too_long_wait() -> None:
+            await test_object_connection.looong_method()
+
+        t1 = loop.create_task(too_long_wait())
+
+        self.bus.close()
+
+        try:
+            await wait_for(t1, timeout=1)
+        except SdBusLibraryError:
+            ...
+
+        self.assertRaises(SdBusLibraryError, t1.result)
+
+    async def test_singal_queue_wildcard_match(self) -> None:
+        test_object, test_object_connection = initialize_object()
+
+        message_queue = await self.bus.get_signal_queue_async(
+            'org.example.test',
+            None, None, None)
+
+        test_object.test_signal.emit(('test', 'signal'))
+
+        message = await wait_for(message_queue.get(), timeout=1)
+        self.assertEqual(message.member,
+                         test_object.test_signal.dbus_signal.signal_name)
+
+    async def test_class_with_string_subclass_parameter(self) -> None:
+        from enum import Enum
+
+        class InterfaceNameEnum(str, Enum):
+            FOO = 'org.example.foo'
+            BAR = 'org.example.bar'
+
+        class ObjectPathEnum(str, Enum):
+            FOO = '/foo'
+            BAR = '/bar'
+
+        class EnumedInterfaceAsync(
+            DbusInterfaceCommonAsync,
+            interface_name=InterfaceNameEnum.BAR,
+        ):
+
+            @dbus_property_async('s')
+            def hello_world(self) -> str:
+                return 'Hello World!'
+
+        test_object = EnumedInterfaceAsync()
+        test_object.export_to_dbus(ObjectPathEnum.FOO)
+
+    async def test_name_validations(self) -> None:
+        if not __debug__:
+            raise SkipTest('Assertions are not enabled')
+
+        try:
+            is_interface_name_valid('org.test')
+        except NotImplementedError:
+            raise SkipTest('Validation functions not available')
+
+        def test_bad_interface_name() -> None:
+            class BadInterfaceName(
+                DbusInterfaceCommonAsync,
+                interface_name='0.test',
+            ):
+                ...
+
+        self.assertRaisesRegex(
+            AssertionError,
+            '^Invalid interface name',
+            test_bad_interface_name,
+        )
+
+        def test_bad_method_name() -> None:
+            class BadMethodName(
+                DbusInterfaceCommonAsync,
+                interface_name='org.example',
+            ):
+                @dbus_method_async(
+                    result_signature='s',
+                    method_name='🤫',
+                )
+                async def test(self) -> str:
+                    return 'test'
+
+        self.assertRaisesRegex(
+            AssertionError,
+            '^Invalid method name',
+            test_bad_method_name,
+        )
+
+        def test_bad_property_name() -> None:
+            class BadPropertyName(
+                DbusInterfaceCommonAsync,
+                interface_name='org.example',
+            ):
+                @dbus_property_async(
+                    property_signature='s',
+                    property_name='🤫',
+                )
+                def test(self) -> str:
+                    return 'test'
+
+        self.assertRaisesRegex(
+            AssertionError,
+            '^Invalid property name',
+            test_bad_property_name,
+        )
+
+        def test_bad_signal_name() -> None:
+            class BadSignalName(
+                DbusInterfaceCommonAsync,
+                interface_name='org.example',
+            ):
+                @dbus_signal_async(
+                    signal_signature='s',
+                    signal_name='🤫',
+                )
+                def test(self) -> str:
+                    raise NotImplementedError
+
+        self.assertRaisesRegex(
+            AssertionError,
+            '^Invalid signal name',
+            test_bad_signal_name,
+        )
