@@ -22,14 +22,13 @@ from __future__ import annotations
 
 from unittest import main
 
+from sdbus.unittest import IsolatedDbusTestCase
 from sdbus_block.dbus_daemon import FreedesktopDbus
 
 from sdbus import DbusPropertyReadOnlyError
 
-from .common_test_util import TempDbusTest
 
-
-class TestSync(TempDbusTest):
+class TestSync(IsolatedDbusTestCase):
 
     def test_sync(self) -> None:
         self.bus.request_name('org.example.test', 0)
